package display

import (
	"context"
	"fmt"
	"image"
	"log"
	"math/rand"
	"time"

	"gitea.unprism.ru/yotia/display-test/pkg/mt12232a"
)

var _ = rand.Int() // TMP for import to exist

type displayMt12232a struct {
	logger *log.Logger

	// GPIO pins
	dev mt12232a.Device
}

func newMt12232a(logger *log.Logger) (Display, error) {
	dev, err := mt12232a.New(log.New(logger.Writer(), "display-mt12864 : ", log.LstdFlags))
	if err != nil {
		return nil, fmt.Errorf("mt12864 create: %w", err)
	}
	// Allocate bits
	bits := make([][]byte, 32)
	for i := 0; i < 32; i++ {
		bits[i] = make([]byte, 122)
	}

	// if err := dev.PowerOn(); err != nil {
	// 	return nil, fmt.Errorf("power on: %w", err)
	// }

	// Setup submit goroutine
	d := displayMt12232a{
		logger: logger,
		dev:    dev,
	}

	// Temp debug draw
	//if st0, st1 := d.dev.ReadStatus(0), d.dev.ReadStatus(1); false { //st0&0x20 == 0 && st1&0x20 == 0 {
	//	d.logger.Println("Display is already on")
	//} else {
	d.status("Setup start")
	if err := d.dev.Reset(); err != nil {
		return nil, fmt.Errorf("reset: %w", err)
	}
	d.status("After reset")
	time.Sleep(200 * time.Millisecond)
	if st0, st1 := d.dev.ReadStatus(0), d.dev.ReadStatus(1); st0 == 0 && st1 == 0 {
		return nil, fmt.Errorf("No status response from dysplay")
	}
	if err := d.powerOn(); err != nil {
		return nil, fmt.Errorf("power on: %w", err)
	}
	d.status("Setup end")
	return &d, nil
}

func (d *displayMt12232a) status(label string) {
	//d.logger.Printf("STATUS %s -- L: %08b R: %08b\n", label, d.dev.ReadStatus(0)&0xFF, d.dev.ReadStatus(1)&0xFF)
}

func (d *displayMt12232a) powerOn() error {
	// Reset
	if err := d.dev.WriteCode(0, 0xE2); err != nil {
		return fmt.Errorf("reset: %w", err)
	}
	if err := d.dev.WriteCode(1, 0xE2); err != nil {
		return fmt.Errorf("reset: %w", err)
	}
	// ReadModifyWrite off
	if err := d.dev.WriteCode(0, 0xEE); err != nil {
		return fmt.Errorf("RMW off: %w", err)
	}
	if err := d.dev.WriteCode(1, 0xEE); err != nil {
		return fmt.Errorf("RMW off: %w", err)
	}
	// Turn on common mode
	if err := d.dev.WriteCode(0, 0xA4); err != nil {
		return fmt.Errorf("turn on common mode: %w", err)
	}
	if err := d.dev.WriteCode(1, 0xA4); err != nil {
		return fmt.Errorf("turn on common mode: %w", err)
	}
	// Multiplex 1/32
	if err := d.dev.WriteCode(0, 0xA9); err != nil {
		return fmt.Errorf("multiplex 1/32: %w", err)
	}
	if err := d.dev.WriteCode(1, 0xA9); err != nil {
		return fmt.Errorf("multiplex 1/32: %w", err)
	}
	// Top line to 0
	if err := d.dev.WriteCode(0, 0xC0); err != nil {
		return fmt.Errorf("top line to 0: %w", err)
	}
	if err := d.dev.WriteCode(1, 0xC0); err != nil {
		return fmt.Errorf("top line to 0: %w", err)
	}
	// Invert scan RAM
	if err := d.dev.WriteCode(0, 0xA1); err != nil {
		return fmt.Errorf("inver scan RAM: %w", err)
	}
	if err := d.dev.WriteCode(1, 0xA0); err != nil {
		return fmt.Errorf("inver scan RAM: %w", err)
	}
	// Display on
	if err := d.dev.WriteCode(0, 0xAF); err != nil {
		return fmt.Errorf("display on: %w", err)
	}
	if err := d.dev.WriteCode(1, 0xAF); err != nil {
		return fmt.Errorf("display on: %w", err)
	}

	//time.Sleep(time.Second)

	// Check that crystals are turned on

	// The same but with error
	if d.dev.ReadStatus(0)&0x20 != 0 {
		return fmt.Errorf("Left cristal is off")
	}
	if d.dev.ReadStatus(1)&0x20 != 0 {
		return fmt.Errorf("Right cristal is off")
	}
	return nil
}

func (d *displayMt12232a) Test(ctx context.Context) error {
	d.dev.WriteCode(0, 0xB8)
	d.dev.WriteCode(0, 0x14)
	d.dev.WriteData(0, 0xAE)
	start := time.Now()

	d.status("4")
	for p := 0; p < 4; p++ {
		// First

		// d.dev.WriteCode(0, byte(p)|0xB8)
		// d.dev.WriteCode(0, 0x13)
		// for c := 0; c < 61; c++ {
		// d.dev.WriteData(0, 0)
		// }
		// d.dev.WriteCode(1, byte(p)|0xB8)
		// d.dev.WriteCode(1, 0x00)
		// for c := 0; c < 61; c++ {
		// d.dev.WriteData(1, 0)
		// }
		//
		// Second
		d.dev.WriteCode(0, byte(p)|0xB8)
		d.dev.WriteCode(0, 0x13)
		for c := 0; c < 61; c++ {
			//d.dev.WriteDatas(0, []byte{byte(rand.Int() % 2), byte(rand.Int() % 2), byte(rand.Int() % 2), byte(rand.Int() % 2), byte(rand.Int() % 2), byte(rand.Int() % 2), byte(rand.Int() % 2), byte(rand.Int() % 2)})
			//d.dev.WriteData(0, byte(0xF0^(0xFF*(c%2))))
			d.dev.WriteData(0, byte(rand.Int()))
			// d.dev.WriteData(0, 0xFF)
			//d.dev.WriteData(0, byte(p*122+c))
			//time.Sleep(2 * time.Millisecond)
		}
		d.dev.WriteCode(1, byte(p)|0xB8)
		d.dev.WriteCode(1, 0x00)
		for c := 0; c < 61; c++ {
			//d.dev.WriteDatas(1, []byte{byte(rand.Int() % 2), byte(rand.Int() % 2), byte(rand.Int() % 2), byte(rand.Int() % 2), byte(rand.Int() % 2), byte(rand.Int() % 2), byte(rand.Int() % 2), byte(rand.Int() % 2)})
			d.dev.WriteData(1, byte(rand.Int()))
			// d.dev.WriteData(1, 0xFF)
			//d.dev.WriteData(1, byte(4*int(p)+c))
			//d.dev.WriteData(1, byte(p*122+c+61))
			//d.dev.WriteData(1, byte(rand.Int())>>1)
			//time.Sleep(2 * time.Millisecond)
		}
	}
	d.status("9")
	end := time.Now()
	d.logger.Println(end.Sub(start))
	// for p := byte(0); p < 8; p++ {
	// 	d.dev.WriteCodeL(p | 0xB8)
	// 	d.dev.WriteCodeL(0x40)
	// 	for c := 0; c < 64; c++ {
	// 		d.dev.WriteDataL(Logo128[p][c])
	// 	}
	// 	d.dev.WriteCodeR(p | 0xB8)
	// 	d.dev.WriteCodeR(0x40)
	// 	for c := 64; c < 128; c++ {
	// 		d.dev.WriteDataR(Logo128[p][c])
	// 	}
	// }
	//d.dev.WriteCodeL(0xAF) // Display on
	//d.dev.WriteCodeR(0xAF) // Display on
	return nil

}

func (d *displayMt12232a) GetBounds() image.Rectangle {
	return image.Rect(0, 0, 122, 32)
}

func (d *displayMt12232a) Flush(img *image.Gray) error {
	for p := byte(0); p < 4; p++ {
		d.dev.WriteCode(0, (3-p)|0xB8)
		d.dev.WriteCode(0, 0x13)
		for c := 0; c < 61; c++ {
			d.dev.WriteDatas(0, []byte{
				img.Pix[int(p<<3+7)*122+c],
				img.Pix[int(p<<3+6)*122+c],
				img.Pix[int(p<<3+5)*122+c],
				img.Pix[int(p<<3+4)*122+c],
				img.Pix[int(p<<3+3)*122+c],
				img.Pix[int(p<<3+2)*122+c],
				img.Pix[int(p<<3+1)*122+c],
				img.Pix[int(p<<3+0)*122+c],
			})
		}
		d.dev.WriteCode(1, (3-p)|0xB8)
		d.dev.WriteCode(1, 0x00)
		for c := 61; c < 122; c++ {
			d.dev.WriteDatas(1, []byte{
				img.Pix[int(p<<3+7)*122+c],
				img.Pix[int(p<<3+6)*122+c],
				img.Pix[int(p<<3+5)*122+c],
				img.Pix[int(p<<3+4)*122+c],
				img.Pix[int(p<<3+3)*122+c],
				img.Pix[int(p<<3+2)*122+c],
				img.Pix[int(p<<3+1)*122+c],
				img.Pix[int(p<<3+0)*122+c],
			})
		}
	}
	return nil
}

func (d *displayMt12232a) Close() error {

	return d.dev.Close()
}
